# -*- coding: utf-8 -*-
from setuphelpers import *
from typing import Union
import shutil
import json


# Define depends_package if you want to remove source when package is installed
depends_package = ""
sources_dir_name = "sources"


def install():
    sources_location = makepath(WAPT.wapt_base_dir, "public", "sources")
    sources_list = list(set([os.path.basename(s) for s in glob.glob(makepath(sources_dir_name, "*"))]))
    sources_dict = {"sources_list": sources_list}
    mkdirs(get_private_persistent_package_dir())
    json_write_file(get_private_persistent_package_file("sources.json"), sources_dict)

    if not sources_list:
        error("You have not provided any sources to copy. Please run the update_package() first or place them manually in the sources directory.")
    print(f"Sources will be placed in: {sources_location}")
    mkdirs(sources_location)
    for source in sources_list:
        source_src = makepath(sources_dir_name, source)
        source_dst = makepath(sources_location, source)
        if isdir(source_src):
            print(f"Copying: {source_src} directory to: {source_dst}")
            if running_as_system():
                if isdir(source_dst):
                    remove_tree(source_dst)
                shutil.move(source_src, source_dst)
            else:
                copytree2(source_src, source_dst)
        if isfile(source_src):
            print(f"Copying: {source_src} file to: {source_dst}")
            if running_as_system():
                if isfile(source_dst):
                    remove_file(source_dst)
                shutil.move(source_src, source_dst)
            else:
                filecopyto(source_src, source_dst)


def audit():
    audit_status = "OK"

    if depends_package:
        sources_dict = json_load_file(get_private_persistent_package_file("sources.json"))
        if sources_dict.get("deleted_by", ""):
            print(f'Sources have already been uninstalled since {sources_dict["deleted_by"]} was successfully installed.')
        else:
            depends_package_is_installed = WAPT.is_installed(depends_package)
            if depends_package_is_installed:
                # WAPT.is_installed("tis-notepadplusplus").asrequirement()
                print(f"Removing sources since {depends_package_is_installed.package} is installed.")
                uninstall()

                sources_dict.update({"deleted_by": depends_package_is_installed.asrequirement()})
                json_write_file(get_private_persistent_package_file("sources.json"), sources_dict)
    else:
        print(
            "You did not define depends_package. sources will remain until the package is removed (note that if you remove this package depends_package may also be removed)."
        )

    return audit_status


def uninstall():
    sources_location = makepath(WAPT.wapt_base_dir, "public", "sources")
    sources_list = json_load_file(get_private_persistent_package_file("sources.json"))["sources_list"]
    for source in sources_list:
        source_dst = makepath(sources_location, source)
        if isdir(source_dst):
            print(f"Removing: {source_dst} directory.")
            remove_tree(source_dst)
        if isfile(source_dst):
            print(f"Removing: {source_dst} file.")
            remove_file(source_dst)
    if dir_is_empty(sources_location):
        print(f"Removing: {sources_location} sources directory since it is empty.")
        remove_tree(sources_location)


def get_private_persistent_package_dir():
    if control.package_uuid:
        if WAPT.is_installed(control.package) is not None and control.package_uuid == WAPT.is_installed(control.package)["package_uuid"]:
            return makepath(WAPT.persistent_root_dir, control.package_uuid)
    return makepath(os.getcwd(), "WAPT", "persistent")


def get_private_persistent_package_file(file_name):
    file_name = os.path.basename(file_name)
    return makepath(get_private_persistent_package_dir(), file_name)


def json_load_file(json_file: str, encoding: str = "utf-8") -> Union[list, dict]:
    """Load content from a JSON file.

    Args:
        json_file: Path to the JSON file.
        encoding: File encoding.

    Returns:
        Loaded JSON content as a dictionary or a list.
    """
    with open(json_file, encoding=encoding) as read_file:
        return json.load(read_file)


def json_write_file(json_file: str, data: Union[list, dict], indent: int = 4, sort_keys: bool = False, encoding: str = "utf-8", newline: str = "\n"):
    """Write dictionary or list to a JSON file.

    Args:
        json_file: Path to the JSON file.
        data: Dictionary or list content to be written.
        indent: Tabulation size for indentation.
        sort_keys: Sort the keys alphabetically or not.
        encoding: File encoding.
        newline: Newline character(s) to use, default is Line Feed (LF).
    """
    with open(json_file, "w", encoding=encoding, newline=newline) as write_file:
        json.dump(data, write_file, sort_keys=sort_keys, indent=indent)
